/**
 * Re-maps a number from one range to another.
 *
 * This function behaves similarly to the Arduino `map` function. It takes a number defined 
 * within a certain range (inclusive) and maps it to a new number within a different range
 * (also inclusive). Optionally, the function can allow the output to exceed the mapped range.
 * The function is especially useful for adapting sensor values or other input data to a specific scale.
 *
 * @param {number} x - The number to re-map. Expected to be within the range of in_min to in_max (inclusive).
 * @param {number} in_min - The lower bound of the current range of x (inclusive).
 * @param {number} in_max - The upper bound of the current range of x (inclusive).
 * @param {number} out_min - The lower bound of the target range (inclusive).
 * @param {number} out_max - The upper bound of the target range (inclusive).
 * @param {boolean} [allowExceed=false] - Optional. If true, allows the output to exceed the specified range.
 * @return {number} The re-mapped number. Constrained within out_min and out_max if allowExceed is false.
 *
 * @example
 * // Map a sensor value from a range of 0-1023 to a range of 0-100
 * let sensorValue = 512;
 * let mappedValue = map(sensorValue, 0, 1023, 0, 100);
 * console.log(mappedValue); // Outputs: 50
 *
 * // Example where output exceeds the range
 * mappedValue = map(sensorValue, 0, 1023, 0, 100, true);
 * console.log(mappedValue); // Output may exceed 100 if sensorValue > 1023
 */
export function map(x, in_min, in_max, out_min, out_max, allowExceed = false) {
    let mapped = (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;

    if (!allowExceed) {
        mapped = clamp(mapped, out_min, out_max);
    }

    return mapped;
}


/**
 * Clamps a number between two inclusive bounds.
 *
 * This function forces a number to be within a specified range. If the number is less than
 * the lower bound, the lower bound is returned. If the number is greater than the upper bound,
 * the upper bound is returned. Otherwise, the original number is returned. Both the lower and
 * upper bounds are inclusive.
 *
 * @param {number} value - The number to clamp.
 * @param {number} minVal - The lower bound of the clamping range (inclusive).
 * @param {number} maxVal - The upper bound of the clamping range (inclusive).
 * @return {number} The clamped value.
 *
 * @example
 * console.log(clamp(7, 1, 5)); // Outputs: 5
 * console.log(clamp(2, 1, 5)); // Outputs: 2
 */
function clamp(value, minVal, maxVal) {
    return Math.max(minVal, Math.min(maxVal, value));
}
